{-|
Module      : Test.API.Notifications
Description : Tests for the notification mechanism (central exchange, dispatcher, websockets)
Copyright   : (c) CNRS, 2017-Present
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX

-}

{-# OPTIONS_GHC -Wno-orphans  #-}

{-# LANGUAGE ScopedTypeVariables #-}

    
module Test.API.Notifications (
    tests
  ) where

import Control.Concurrent (threadDelay)
import Control.Concurrent.Async (withAsync)
import Control.Concurrent.STM.TChan
import Control.Exception.Safe qualified as Exc
import Control.Monad (void)
import Control.Monad.STM (atomically)
import Data.Aeson qualified as Aeson
import Data.ByteString qualified as BS
import Data.Maybe (isJust)
import Gargantext.Core.Notifications.CentralExchange qualified as CE
import Gargantext.Core.Notifications.CentralExchange.Types qualified as CET
import Gargantext.Core.Notifications.Dispatcher qualified as D
import Gargantext.Core.Notifications.Dispatcher.Types qualified as DT
import Gargantext.Core.Config.Types (NotificationsConfig(..))
import Network.WebSockets qualified as WS
import Prelude
import Test.API.Setup (withTestDBAndNotifications)
import Test.Hspec
import Test.Instances ()
import Test.Utils.Notifications



tests :: NotificationsConfig -> D.Dispatcher -> Spec
tests nc dispatcher = sequential $ aroundAll (withTestDBAndNotifications dispatcher) $ do
  describe "Dispatcher, Central Exchange, WebSockets" $ do
    it "simple WS notification works" $ \((testEnv, port), _) -> do
      let topic = DT.UpdateTree 0
      tchan <- newTChanIO :: IO (TChan (Maybe DT.Notification))
      -- setup a websocket connection
      let wsConnect =
            withWSConnection ("127.0.0.1", port) $ \conn -> do
              -- We wait a bit before the server settles
              threadDelay (100 * millisecond)
              WS.sendTextData conn $ Aeson.encode (DT.WSSubscribe topic)
              d <- WS.receiveData conn
              let dec = Aeson.decode d :: Maybe DT.Notification
              atomically $ writeTChan tchan dec
            
      -- wait a bit to settle
      threadDelay (100 * millisecond)

      withAsync wsConnect $ \_a -> do
        -- wait a bit to connect
        threadDelay (500 * millisecond)

        let nodeId = 0
        CE.notify nc $ CET.UpdateTreeFirstLevel nodeId
  
        md <- atomically $ readTChan tchan
  
        md `shouldBe` Just (DT.NUpdateTree nodeId)
